@@ -25,12 +25,14 @@
  */
 
 #include <config.h>
+#include <string.h>
 #include "nautilus-file-operations.h"
 
 #include "nautilus-file-operations-progress.h"
 #include "nautilus-lib-self-check-functions.h"
 
 #include <eel/eel-glib-extensions.h>
+#include <eel/eel-pango-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
@@ -86,7 +88,7 @@ transfer_info_new (GtkWidget *parent_view)
 	result = g_new0 (TransferInfo, 1);
 	result->parent_view = parent_view;
 	
-	eel_nullify_when_destroyed (&result->parent_view);
+	eel_add_weak_pointer (&result->parent_view);
 	
 	return result;
 }
@@ -94,7 +96,7 @@ transfer_info_new (GtkWidget *parent_view)
 static void
 transfer_info_destroy (TransferInfo *transfer_info)
 {
-	eel_nullify_cancel (&transfer_info->parent_view);
+	eel_remove_weak_pointer (&transfer_info->parent_view);
 	
 	if (transfer_info->progress_dialog != NULL) {
 		nautilus_file_operations_progress_done (transfer_info->progress_dialog);
@@ -194,76 +196,60 @@ icon_position_iterator_get_next (IconPositionIterator *position_iterator,
 	return TRUE;
 }
 
-#if GNOME2_CONVERSION_COMPLETE
-
-static GdkFont *
-get_label_font (void)
-{
-	GtkWidget *label;
-	GtkStyle *style;
-	GdkFont *font;
-
-	/* FIXME: Does this really pick up the correct font if the rc
-	 * file has a special case for the particular dialog we are
-	 * preparing?
-	 */
-
-	label = gtk_label_new ("");
-	style = gtk_widget_get_style (label);
-	font = style->font;
-	gdk_font_ref (font);
-	gtk_object_sink (GTK_OBJECT (label));
-
-	return font;
-}
-
-#endif
-
 static char *
-ellipsize_string_for_dialog (const char *str)
+ellipsize_string_for_dialog (PangoContext *context, const char *str)
 {
-	char *result;
-#ifdef GNOME2_CONVERSION_COMPLETE
-	GdkFont *font;
 	int maximum_width;
+	char *result;
+	PangoLayout *layout;
+	PangoFontMetrics *metrics;
 
-	/* get a nice length to ellipsize to, based on the font */
-	font = get_label_font ();
-	maximum_width = gdk_string_width (font, "MMMMMMMMMMMMMMMMMMMMMM");
+	layout = pango_layout_new (context);
 
-	result = eel_string_ellipsize (str, font, maximum_width, EEL_ELLIPSIZE_MIDDLE);
-	gdk_font_unref (font);
-#else
-	result = g_strdup (str);
-#endif
+	metrics = pango_context_get_metrics (
+		context, pango_context_get_font_description (context), NULL);
+
+	maximum_width = pango_font_metrics_get_approximate_char_width (metrics) * 25 / PANGO_SCALE;
+
+	pango_font_metrics_unref (metrics);
+
+	eel_pango_layout_set_text_ellipsized (
+		layout, str, maximum_width, EEL_ELLIPSIZE_MIDDLE);
+
+	result = g_strdup (pango_layout_get_text (layout));
+
+	g_object_unref (layout);
 
 	return result;
 }
 
 static char *
-format_and_ellipsize_uri_for_dialog (const char *uri)
+format_and_ellipsize_uri_for_dialog (GtkWidget *context, const char *uri)
 {
 	char *unescaped, *result;
 
 	unescaped = eel_format_uri_for_display (uri);
-	result = ellipsize_string_for_dialog (unescaped);
+	result = ellipsize_string_for_dialog (
+		gtk_widget_get_pango_context (context), unescaped);
 	g_free (unescaped);
 
 	return result;
 }
 
 static char *
-extract_and_ellipsize_file_name_for_dialog (const char *uri)
+extract_and_ellipsize_file_name_for_dialog (GtkWidget *context, const char *uri)
 {
-	const char *last_part;
+	char *basename;
 	char *unescaped, *result;
 	
-	last_part = g_basename (uri);
-	g_return_val_if_fail (last_part != NULL, NULL);
+	basename = g_path_get_basename (uri);
+	g_return_val_if_fail (basename != NULL, NULL);
 
-	unescaped = gnome_vfs_unescape_string_for_display (last_part);
-	result = ellipsize_string_for_dialog (unescaped);
+	unescaped = gnome_vfs_unescape_string_for_display (basename);
+	result = ellipsize_string_for_dialog (
+		gtk_widget_get_pango_context (context), unescaped);
 	g_free (unescaped);
+	g_free (basename);
 
 	return result;
 }
@@ -278,59 +264,6 @@ parent_for_error_dialog (TransferInfo *transfer_info)
 	return transfer_info->parent_view;
 }
 
-static void
-fit_rect_on_screen (GdkRectangle *rect)
-{
-	if (rect->x + rect->width > gdk_screen_width ()) {
-		rect->x = gdk_screen_width () - rect->width;
-	}
-
-	if (rect->y + rect->height > gdk_screen_height ()) {
-		rect->y = gdk_screen_height () - rect->height;
-	}
-
-	if (rect->x < 0) {
-		rect->x = 0;
-	}
-
-	if (rect->y < 0) {
-		rect->y = 0;
-	}
-}
-
-static void
-center_dialog_over_rect (GtkWindow *window, GdkRectangle rect)
-{
-	g_return_if_fail (GTK_WINDOW (window) != NULL);
-
-	fit_rect_on_screen (&rect);
-
-	gtk_widget_set_uposition (GTK_WIDGET (window), 
-		rect.x + rect.width / 2 
-			- GTK_WIDGET (window)->allocation.width / 2,
-		rect.y + rect.height / 2
-			- GTK_WIDGET (window)->allocation.height / 2);
-}
-
-static void
-center_dialog_over_window (GtkWindow *window, GtkWindow *over)
-{
-	GdkRectangle rect;
-	int x, y, w, h;
-
-	g_return_if_fail (GTK_WINDOW (window) != NULL);
-	g_return_if_fail (GTK_WINDOW (over) != NULL);
-
-	gdk_window_get_root_origin (GTK_WIDGET (over)->window, &x, &y);
-	gdk_window_get_size (GTK_WIDGET (over)->window, &w, &h);
-	rect.x = x;
-	rect.y = y;
-	rect.width = w;
-	rect.height = h;
-
-	center_dialog_over_rect (window, rect);
-}
-
 static void
 handle_response_callback (GtkDialog *dialog, int response, TransferInfo *transfer_info)
 {
@@ -362,7 +295,7 @@ create_transfer_dialog (const GnomeVFSXferProgressInfo *progress_info,
 	g_signal_connect (transfer_info->progress_dialog,
 			  "response",
 			  G_CALLBACK (handle_response_callback),
-			  NULL);
+			  transfer_info);
 	g_signal_connect (transfer_info->progress_dialog,
 			  "close",
 			  G_CALLBACK (handle_close_callback),
@@ -370,8 +303,9 @@ create_transfer_dialog (const GnomeVFSXferProgressInfo *progress_info,
 
 	/* Make the progress dialog show up over the window we are copying into */
 	if (transfer_info->parent_view != NULL) {
-		center_dialog_over_window (GTK_WINDOW (transfer_info->progress_dialog), 
-					   GTK_WINDOW (gtk_widget_get_toplevel (transfer_info->parent_view)));
+		gtk_window_set_transient_for (
+			GTK_WINDOW (transfer_info->progress_dialog), 
+			GTK_WINDOW (gtk_widget_get_toplevel (transfer_info->parent_view)));
 	}
 
 	gtk_widget_show (GTK_WIDGET (transfer_info->progress_dialog));
@@ -871,12 +805,14 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 
 		if (progress_info->source_name != NULL) {
 			formatted_source_name = format_and_ellipsize_uri_for_dialog
-				(progress_info->source_name);
+				(parent_for_error_dialog (transfer_info),
+				 progress_info->source_name);
 		}
 
 		if (progress_info->target_name != NULL) {
 			formatted_target_name = format_and_ellipsize_uri_for_dialog
-				(progress_info->target_name);
+				(parent_for_error_dialog (transfer_info),
+				 progress_info->target_name);
 		}
 
 		error_kind = ERROR_OTHER;
@@ -1046,18 +982,12 @@ is_special_link (const char *uri)
 	gboolean is_special;
 
 	local_path = gnome_vfs_get_local_path_from_uri (uri);
-
-	/* FIXME: This should use some API to check if the file is a
-	 * link. Normally we use the MIME type. As things stand, this
-	 * will read files and try to parse them as XML, which could
-	 * result in a lot of output to the console, since the XML
-	 * parser reports errors directly there.
-	 */
-	is_special = local_path != NULL
-		&& nautilus_link_local_get_link_type (local_path) != NAUTILUS_LINK_GENERIC;
-	
+	if (local_path == NULL) {
+		return FALSE;
+	}
+	is_special = nautilus_link_local_is_special_link (local_path);
 	g_free (local_path);
-	
+
 	return is_special;
 }
 
@@ -1071,7 +1001,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 	/* Handle special case files such as Trash, mount links and home directory */	
 	if (is_special_link (progress_info->target_name)) {
 		formatted_name = extract_and_ellipsize_file_name_for_dialog
-			(progress_info->target_name);
+			(parent_for_error_dialog (transfer_info),
+			 progress_info->target_name);
 		
 		if (transfer_info->kind == TRANSFER_MOVE) {
 			text = g_strdup_printf (_("\"%s\" could not be moved to the new location, "
@@ -1097,7 +1028,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 	}
 	
 	/* transfer conflict, prompt the user to replace or skip */
-	formatted_name = format_and_ellipsize_uri_for_dialog (progress_info->target_name);
+	formatted_name = format_and_ellipsize_uri_for_dialog (
+		parent_for_error_dialog (transfer_info), progress_info->target_name);
 	text = g_strdup_printf (_("File \"%s\" already exists.\n\n"
 				  "Would you like to replace it?"), 
 				formatted_name);
@@ -1533,8 +1465,8 @@ handle_transfer_duplicate (GnomeVFSXferProgressInfo *progress_info,
 			(progress_info->duplicate_name,
 			 progress_info->duplicate_count);
 		break;
-
 	default:
+		break;
 		/* For all other cases we use the name as-is. */
 	}
 
@@ -1954,7 +1886,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 				 FALSE,
 				 _("You cannot copy items into the Trash."), 
 				 _("Can't Copy to Trash"),
-				 GNOME_STOCK_BUTTON_OK, NULL, NULL);			
+				 GTK_STOCK_OK, NULL);
 			result = GNOME_VFS_ERROR_NOT_PERMITTED;
 		}
 	}
@@ -1976,19 +1908,19 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			    	is_desktop_trash_link = vfs_uri_is_special_link (uri);
 
 				eel_run_simple_dialog
-					(parent_view, 
+					(parent_view,
 					 FALSE,
-					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0) 
+					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0)
 						 ? (is_desktop_trash_link
 						    ? _("The Trash must remain on the desktop.")
 						    : _("You cannot move this trash folder."))
 						 : (is_desktop_trash_link
 						    ? _("You cannot copy the Trash.")
-						    : _("You cannot copy this trash folder.")), 
+						    : _("You cannot copy this trash folder.")),
 					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0)
 						 ? _("Can't Change Trash Location")
 						 : _("Can't Copy Trash"),
-					 GNOME_STOCK_BUTTON_OK, NULL, NULL);			
+					 GTK_STOCK_OK, NULL, NULL);
 
 				result = GNOME_VFS_ERROR_NOT_PERMITTED;
 				break;
@@ -2015,7 +1947,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0) 
 					 ? _("Can't Move Into Self")
 					 : _("Can't Copy Into Self"),
-					 GNOME_STOCK_BUTTON_OK, NULL, NULL);			
+					 GTK_STOCK_OK, NULL, NULL);			
 
 				result = GNOME_VFS_ERROR_NOT_PERMITTED;
 				break;
@@ -2028,7 +1960,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 					 FALSE,
 					 _("You cannot copy a file over itself."), 
 					 _("Can't Copy Over Self"), 
-					 GNOME_STOCK_BUTTON_OK, NULL, NULL);			
+					 GTK_STOCK_OK, NULL, NULL);			
 
 				result = GNOME_VFS_ERROR_NOT_PERMITTED;
 				break;
@@ -2110,7 +2042,7 @@ new_folder_transfer_callback (GnomeVFSAsyncHandle *handle,
 	switch (progress_info->phase) {
 
 	case GNOME_VFS_XFER_PHASE_COMPLETED:
-		eel_nullify_cancel (&state->parent_view);
+		eel_remove_weak_pointer (&state->parent_view);
 		g_free (state);
 		return 0;
 
@@ -2171,7 +2103,7 @@ nautilus_file_operations_new_folder (GtkWidget *parent_view,
 	state->done_callback = done_callback;
 	state->data = data;
 	state->parent_view = parent_view;
-	eel_nullify_when_destroyed (&state->parent_view);
+	eel_add_weak_pointer (&state->parent_view);
 
 	/* pass in the target directory and the new folder name as a destination URI */
 	parent_uri = gnome_vfs_uri_new (parent_dir);
@@ -2288,7 +2220,7 @@ confirm_empty_trash (GtkWidget *parent_view)
 		  "all of the items in the Trash?"),
 		_("Delete Trash Contents?"),
 		_("Empty"),
-		GNOME_STOCK_BUTTON_CANCEL,
+		GTK_STOCK_CANCEL,
 		parent_window);
 
 	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
