@@ -36,9 +36,6 @@ G_BEGIN_DECLS
 /* How wide the sidebar is (or how wide it will be when expanded) */
 #define NAUTILUS_PREFERENCES_SIDEBAR_WIDTH  			"preferences/sidebar_width"
 
-/* Text fields */
-#define NAUTILUS_PREFERENCES_USE_EMACS_SHORTCUTS                "preferences/use_emacs_shortcuts"
-
 /* Window options */
 #define NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW			"preferences/window_always_new"
 
@@ -77,18 +74,18 @@ G_BEGIN_DECLS
 
 /* Content fonts */
 #define NAUTILUS_PREFERENCES_DEFAULT_FONT			"preferences/default_font"
-#define NAUTILUS_PREFERENCES_DEFAULT_SMOOTH_FONT		"preferences/default_smooth_font"
 #define NAUTILUS_PREFERENCES_DEFAULT_FONT_SIZE			"preferences/default_font_size"
 
 /* Single/Double click preference  */
 #define NAUTILUS_PREFERENCES_CLICK_POLICY			"preferences/click_policy"
 
+/* Xalf settings */
+#define NAUTILUS_PREFERENCES_XALF_ENABLED			"preferences/xalf/enabled"
+#define NAUTILUS_PREFERENCES_XALF_OPTIONS			"preferences/xalf/options"
+
 /* Activating executable text files */
 #define NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION		"preferences/executable_text_activation"
 
-/* Smooth graphics mode (smoother but slower) */
-#define NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE		"preferences/smooth_graphics_mode"
-
 /* Which views should be displayed for new windows */
 #define NAUTILUS_PREFERENCES_START_WITH_TOOLBAR			"preferences/start_with_toolbar"
 #define NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR		"preferences/start_with_location_bar"
@@ -121,7 +118,6 @@ enum
 #define NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE	"icon-view/default_zoom_level_font_size"
 #define NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_MANUAL_LAYOUT	"icon-view/default_use_manual_layout"
 #define NAUTILUS_PREFERENCES_ICON_VIEW_FONT				"icon-view/font"
-#define NAUTILUS_PREFERENCES_ICON_VIEW_SMOOTH_FONT			"icon-view/smooth_font"
 
 /* The icon view uses 2 variables to store the sort order and
  * whether to use manual layout.  However, the UI for these
@@ -149,8 +145,6 @@ enum
 /* File Indexing */
 #define NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE				"preferences/search_bar_type"
 
-/* searching */
-#define NAUTILUS_PREFERENCES_SEARCH_WEB_URI				"preferences/search_web_uri"			
 enum
 {
 	NAUTILUS_CLICK_POLICY_SINGLE,
@@ -185,9 +179,10 @@ typedef enum
 } NautilusSearchBarMode;
 
 /* Gnome session management */
-#define NAUTILUS_PREFERENCES_ADD_TO_SESSION		"preferences/add_to_session"
+#define NAUTILUS_PREFERENCES_ADD_TO_SESSION		   "preferences/add_to_session"
 
 void nautilus_global_preferences_init                      (void);
+void nautilus_global_preferences_init_with_folder_browsing (void);
 void nautilus_global_preferences_set_default_folder_viewer (const char *iid);
 
 G_END_DECLS
